import asyncio
import unittest

from automon.integrations.requestsWrapper.rest import BaseRestClient

r = BaseRestClient()


class Client(unittest.TestCase):
    def test_get(self):
        self.assertTrue(asyncio.run(r.get('https://1.1.1.1')))
        self.assertTrue(r.requests.get('https://1.1.1.1'))
        self.assertFalse(asyncio.run(r.get('x://127.0.0.1')))


if __name__ == '__main__':
    unittest.main()



import unittest
import asyncio

from automon.integrations.instagram.client_browser import InstagramBrowserClient

c = InstagramBrowserClient(headless=True)
asyncio.run(c.start())


class InstagramClientTest(unittest.TestCase):

    async def test(self):
        if await c.is_running():
            await c.browser.get(c.urls.login_page)

            # user
            login_user = await c.browser.wait_for_xpath(c.xpaths.login_user)
            await c.browser.action_click(login_user, 'user')
            await c.browser.action_type(c.login)

            # password
            password = await c.browser.wait_for_xpath(c.xpaths.login_pass)
            await c.browser.action_click(password, 'password')

            await c.browser.quit()


if __name__ == '__main__':
    unittest.main()



import unittest
import asyncio

from automon.integrations.seleniumWrapper import SeleniumBrowser, ChromeWrapper

browser = SeleniumBrowser()
browser.config.webdriver_wrapper = ChromeWrapper()
browser.config.webdriver_wrapper.enable_defaults()
browser.config.webdriver_wrapper.enable_headless()


class SeleniumClientTest(unittest.TestCase):
    if asyncio.run(browser.run()):

        def test_fake_page(self):
            self.assertFalse(browser.get('http://555.555.555.555'))

        def test_real_page(self):
            if asyncio.run(browser.get('http://1.1.1.1')):
                self.assertTrue(True)

        def test_screenshot_png(self):
            if asyncio.run(browser.get('http://google.com')):
                self.assertTrue(asyncio.run(browser.get_screenshot_as_png()))

        def test_screenshot_base64(self):
            if asyncio.run(browser.get('http://yahoo.com')):
                self.assertTrue(asyncio.run(browser.get_screenshot_as_base64()))

        def test_screenshot_file(self):
            if asyncio.run(browser.get('http://bing.com')):
                self.assertTrue(asyncio.run(browser.save_screenshot()))
                self.assertTrue(asyncio.run(browser.save_screenshot(folder='./')))


if __name__ == '__main__':
    unittest.main()
    browser.quit()



import unittest
import asyncio

from automon.integrations.instagram.client_browser import InstagramBrowserClient

c = InstagramBrowserClient(headless=True)
asyncio.run(c.start())


class InstagramClientTest(unittest.TestCase):

    async def test(self):

        if c.is_running():
            await c.browser.get(c.urls.domain)
            await c.browser.add_cookie_from_base64()
            await c.browser.refresh()
            if await c.is_authenticated():
                self.assertTrue(await c.is_authenticated())
                await c.browser.quit()


if __name__ == '__main__':
    unittest.main()



import unittest
import asyncio

from datetime import datetime

from automon import log
from automon.integrations.sentryio import SentryClient
from automon.integrations.geoip import Geoip

s = SentryClient()


class SentryClientTest(unittest.TestCase):
    async def test_sentry(self):
        if await s.isConnected():
            self.assertTrue(asyncio.run(s.capture_exception(Exception(f'test capture_exception'))))
            self.assertTrue(asyncio.run(s.capture_message(f'test capture_message')))
            # self.assertTrue(asyncio.run(s.capture_event('test capture_event', 'warning')))


if __name__ == '__main__':
    unittest.main()



import unittest
import asyncio

from automon.integrations.seleniumWrapper import SeleniumBrowser, ChromeWrapper

browser = SeleniumBrowser()
browser.config.webdriver_wrapper = ChromeWrapper()
browser.config.webdriver_wrapper.enable_defaults()
browser.config.webdriver_wrapper.enable_headless()


class SeleniumClientTest(unittest.TestCase):
    if asyncio.run(browser.run()):

        def test(self):

            asyncio.run(browser.set_window_size(device_type='web-large'))

            if asyncio.run(browser.get('http://bing.com')):
                self.assertTrue(asyncio.run(browser.save_screenshot()))
                self.assertTrue(asyncio.run(browser.save_screenshot()))
                self.assertTrue(asyncio.run(browser.save_screenshot(folder='./')))

            asyncio.run(browser.quit())


if __name__ == '__main__':
    unittest.main()



import unittest
import asyncio

from automon.integrations.seleniumWrapper import SeleniumBrowser, ChromeWrapper

browser = SeleniumBrowser()
browser.config.webdriver_wrapper = ChromeWrapper()
browser.config.webdriver_wrapper.enable_defaults()
browser.config.webdriver_wrapper.enable_headless()


class SeleniumClientTest(unittest.TestCase):
    if asyncio.run(browser.run()):
        def test(self):
            self.assertTrue(asyncio.run(browser.quit()))


if __name__ == '__main__':
    unittest.main()



import unittest

from automon.helpers.subprocessWrapper import Run
from automon.helpers.subprocessWrapper.exceptions import *

run = Run()


class TestRun(unittest.TestCase):
    def test_pip(self):
        self.assertRaises(
            NotSupportedCommand,
            run.run,
            'ls | grep eric', shell=True, text=True,
        )


if __name__ == '__main__':
    unittest.main()



import random
import asyncio
import datetime
import statistics

from automon import log
from automon.helpers.sleeper import Sleeper
from automon.integrations.seleniumWrapper import SeleniumBrowser, ChromeWrapper

logger = log.logging.getLogger(__name__)
logger.setLevel(log.DEBUG)


class FacebookGroups(object):
    _xpath_about = [
        '/html/body/div[1]/div/div[1]/div/div[3]/div/div/div/div[1]/div[1]/div[3]/div/div/div/div/div/div/div[1]/div/div/div/div/div[2]/a[1]/div[1]/span',
        '/html/body/div[1]/div/div[1]/div/div[3]/div/div/div/div[1]/div[1]/div[3]/div/div/div/div/div/div/div[1]/div/div/div/div/div[1]/div[1]/span',
        '/html/body/div[1]/div/div[1]/div/div[3]/div/div/div[1]/div[1]/div[4]/div/div/div/div/div/div[1]/div/div/div/div/div/div[2]/div[1]',
    ]
    _xpath_popup_close = [
        '/html/body/div[1]/div/div[1]/div/div[5]/div/div/div[1]/div/div[2]/div/div/div/div[1]/div/i',
    ]
    _xpath_content_unavailable = [
        '/html/body/div[1]/div/div[1]/div/div[3]/div/div/div/div[1]/div[1]/div/div/div[1]/div[2]/div[1]/span',
        '/html/body/div[1]/div/div[1]/div/div[3]/div/div/div[1]/div[1]/div/div/div[1]/div[2]/div[1]/span',
    ]
    _xpath_creation_date = [
        '/html/body/div[1]/div/div[1]/div/div[3]/div/div/div/div[1]/div[1]/div/div[2]/div/div/div[4]/div/div/div/div/div/div[3]/div/div/div/div/div/div[2]/div/div[3]/div/div/div[2]/div/div/span',
        '/html/body/div[1]/div/div[1]/div/div[3]/div/div/div/div[1]/div[1]/div[4]/div/div/div/div/div/div[3]/div/div/div/div/div/div[2]/div/div[3]/div/div/div[2]/div/div/span',
        '/html/body/div[1]/div/div[1]/div/div[3]/div/div/div[1]/div[1]/div[4]/div/div/div/div/div/div[3]/div/div/div/div/div/div[2]/div/div[3]/div/div/div[2]/div/div/span',
    ]
    _xpath_history = [
        '/html/body/div[1]/div/div[1]/div/div[3]/div/div/div/div[1]/div[1]/div[4]/div/div/div/div/div/div[1]/div/div/div/div/div/div[2]/div[4]/div/div/div[2]/div/div[2]/span/span',
    ]
    _xpath_title = [
        '/html/body/div[1]/div/div[1]/div/div[3]/div/div/div/div[1]/div[1]/div[1]/div[2]/div/div/div/div/div[1]/div/div/div/div/div/div[1]/h1/span/a',
        '/html/body/div[1]/div/div[1]/div/div[3]/div/div/div[1]/div[1]/div[1]/div[2]/div/div/div/div/div[1]/div/div/div/div/div/div[1]/h1/span/a',
    ]
    _xpath_temporarily_blocked = [
        '/html/body/div[1]/div[2]/div[1]/div/div/div[1]/div/div[2]/h2',
        '/html/body/div[1]/div[2]/div[1]/div/div/div[1]/div/div[2]',
    ]
    _xpath_members = [
        '/html/body/div[1]/div/div[1]/div/div[3]/div/div/div/div[1]/div[1]/div/div[2]/div/div/div[4]/div/div/div/div/div/div[3]/div/div/div/div/div/div[2]/div/div[2]/div/div/div[2]/div/div[1]/span',
        '/html/body/div[1]/div/div[1]/div/div[3]/div/div/div/div[1]/div[1]/div[4]/div/div/div/div/div/div[3]/div/div/div/div/div/div[2]/div/div[2]/div/div/div[2]/div/div[1]/span',
        '/html/body/div[1]/div/div[1]/div/div[3]/div/div/div[1]/div[1]/div[4]/div/div/div/div/div/div[3]/div/div/div/div/div/div[2]/div/div[2]/div/div/div[2]/div/div[1]/span',
    ]
    _xpath_must_login = [
        '/html/body/div[1]/div[1]/div[1]/div/div[2]/div/div',
    ]
    _xpath_posts_today = [
        '/html/body/div[1]/div/div[1]/div/div[3]/div/div/div/div[1]/div[1]/div[4]/div/div/div/div/div/div[3]/div/div/div/div/div/div[2]/div/div[1]/div/div/div[2]/div/div[1]/span',
        '/html/body/div[1]/div/div[1]/div/div[3]/div/div/div[1]/div[1]/div[4]/div/div/div/div/div/div[3]/div/div/div/div/div/div[2]/div/div[1]/div/div/div[2]/div/div[1]/span',
    ]
    _xpath_posts_monthly = [
        '/html/body/div[1]/div/div[1]/div/div[3]/div/div/div/div[1]/div[1]/div[4]/div/div/div/div/div/div[3]/div/div/div/div/div/div[2]/div/div[1]/div/div/div[2]/div/div[2]/span',
        '/html/body/div[1]/div/div[1]/div/div[3]/div/div/div[1]/div[1]/div[4]/div/div/div/div/div/div[3]/div/div/div/div/div/div[2]/div/div[1]/div/div/div[2]/div/div[2]/span',
    ]
    _xpath_privacy = [
        '/html/body/div[1]/div/div[1]/div/div[3]/div/div/div/div[1]/div[1]/div[4]/div/div/div/div/div/div[1]/div/div/div/div/div/div[2]/div[2]/div/div/div[2]/div/div[1]/span/span',
        '/html/body/div[1]/div/div[1]/div/div[3]/div/div/div[1]/div[1]/div[4]/div/div/div/div/div/div[1]/div/div/div/div/div/div[2]/div[2]/div/div/div[2]/div/div[1]/span/span',
        '/html/body/div[1]/div/div[1]/div/div[3]/div/div/div[1]/div[1]/div[4]/div/div/div/div/div/div[1]/div/div/div/div/div/div[2]/div[2]/div/div/div[2]/div/div[1]/span/span',
    ]
    _xpath_privacy_details = [
        '/html/body/div[1]/div/div[1]/div/div[3]/div/div/div/div[1]/div[1]/div[4]/div/div/div/div/div/div[1]/div/div/div/div/div/div[2]/div[2]/div/div/div[2]/div/div[2]/span/span',
        '/html/body/div[1]/div/div[1]/div/div[3]/div/div/div[1]/div[1]/div[4]/div/div/div/div/div/div[1]/div/div/div/div/div/div[2]/div[2]/div/div/div[2]/div/div[2]/span/span',
    ]
    _xpath_visible = [
        '/html/body/div[1]/div/div[1]/div/div[3]/div/div/div/div[1]/div[1]/div[4]/div/div/div/div/div/div[1]/div/div/div/div/div/div[2]/div[3]/div/div/div[2]/div/div[2]/span/span',
        '/html/body/div[1]/div/div[1]/div/div[3]/div/div/div[1]/div[1]/div[4]/div/div/div/div/div/div[1]/div/div/div/div/div/div[2]/div[3]/div/div/div[2]/div/div[2]/span/span',
    ]

    def __init__(self, url: str = None):
        """Facebook Groups object

        Depends on Selenium"""
        self._url = url

        self._browser = SeleniumBrowser()

        self._rate_per_minute = 2
        self._rate_counter = []
        self._wait_between_retries = random.choice(range(1, 60))

    async def content_unavailable(self):
        """This content isn't available right now"""

        try:
            text = await self._browser.wait_for_list(self._xpath_content_unavailable)
            text = text.text
            logger.debug(text)
            return text
        except Exception as error:
            message, session, stacktrace = self.error_parsing(error)
            logger.error(str(dict(
                url=self.url,
                message=message,
                session=session,
                stacktrace=stacktrace,
            )))
            await self.screenshot_error()

    async def creation_date(self):

        try:
            text = await self._browser.wait_for_list(self._xpath_creation_date)
            text = text.text
            logger.debug(text)
            return text
        except Exception as error:
            message, session, stacktrace = self.error_parsing(error)
            logger.error(str(dict(
                url=self.url,
                message=message,
                session=session,
                stacktrace=stacktrace,
            )))
            await self.screenshot_error()

    async def creation_date_timestamp(self):
        if await self.creation_date():
            # TODO: convert date to datetime timestamp
            return

    async def current_rate_too_fast(self):
        if await self.average_rate() == 0 or len(self._rate_counter) < 2:
            logger.info(False)
            return False

        if await self.average_rate() < await self.rate_per_minute():
            logger.info(True)
            return True

        return False

    async def rate_per_minute(self) -> int:
        rate = int(60 / self._rate_per_minute)
        logger.info(str(dict(
            seconds=rate,
        )))
        return rate

    async def average_rate(self):
        if self._rate_counter:
            rate = int(statistics.mean(self._rate_counter))
            logger.info(str(dict(
                seconds=rate,
            )))
            return rate
        return 0

    async def history(self):

        try:
            text = await self._browser.wait_for_list(self._xpath_history)
            text = text.text
            logger.debug(text)
            return text
        except Exception as error:
            message, session, stacktrace = self.error_parsing(error)
            logger.error(str(dict(
                url=self.url,
                message=message,
                session=session,
                stacktrace=stacktrace,
            )))
            await self.screenshot_error()

    async def temporarily_blocked(self):
        try:
            text = await self._browser.wait_for_list(
                self._xpath_temporarily_blocked
            )
            text = text.text
            logger.debug(text)
            return text
        except Exception as error:
            message, session, stacktrace = self.error_parsing(error)
            logger.error(str(dict(
                url=self.url,
                message=message,
                session=session,
                stacktrace=stacktrace,
            )))
            await self.screenshot_error()

    async def members(self):

        try:
            # TODO: need to clean up string from members and remove bad chars
            text = await self._browser.wait_for_list(self._xpath_members)
            text = text.text
            logger.debug(text)
            return text
        except Exception as error:
            message, session, stacktrace = self.error_parsing(error)
            logger.error(str(dict(
                url=self.url,
                message=message,
                session=session,
                stacktrace=stacktrace,
            )))
            await self.screenshot_error()

    async def members_count(self):

        if await self.members():
            count = [x for x in await self.members()]
            count = [x for x in count if x in [str(x) for x in range(0, 10)]]
            if count:
                members_count = int(''.join(count)) if count else 0

                logger.debug(members_count)
                return members_count

    async def must_login(self):
        try:
            text = await self._browser.wait_for_list(self._xpath_must_login)
            text = text.text
            logger.debug(text)
            return text
        except Exception as error:
            message, session, stacktrace = self.error_parsing(error)
            logger.error(str(dict(
                url=self.url,
                message=message,
                session=session,
                stacktrace=stacktrace,
            )))
            await self.screenshot_error()

    async def posts_monthly(self):

        try:
            text = await self._browser.wait_for_list(self._xpath_posts_monthly)
            text = text.text
            logger.debug(text)
            return text
        except Exception as error:
            message, session, stacktrace = self.error_parsing(error)
            logger.error(str(dict(
                url=self.url,
                message=message,
                session=session,
                stacktrace=stacktrace,
            )))
            await self.screenshot_error()

    async def posts_monthly_count(self):

        if await self.posts_monthly():
            count = [x for x in await self.posts_monthly()]
            count = [x for x in count if x in [str(x) for x in range(0, 10)]]
            if count:
                posts_monthly_count = int(''.join(count)) if count else 0

                logger.debug(posts_monthly_count)
                return posts_monthly_count

    async def posts_today(self):

        try:
            text = await self._browser.wait_for_list(self._xpath_posts_today)
            text = text.text
            logger.debug(text)
            return text
        except Exception as error:
            message, session, stacktrace = self.error_parsing(error)
            logger.error(str(dict(
                url=self.url,
                message=message,
                session=session,
                stacktrace=stacktrace,
            )))
            await self.screenshot_error()

    async def posts_today_count(self):

        if await self.posts_today():
            count = [x for x in await self.posts_today()]
            count = [x for x in count if x in [str(x) for x in range(0, 10)]]
            if count:
                posts_today_count = int(''.join(count)) if count else 0

                logger.debug(posts_today_count)
                return posts_today_count

    async def privacy(self):

        try:
            text = await self._browser.wait_for_list(self._xpath_privacy)
            text = text.text
            logger.debug(text)
            return text
        except Exception as error:
            message, session, stacktrace = self.error_parsing(error)
            logger.error(str(dict(
                url=self.url,
                message=message,
                session=session,
                stacktrace=stacktrace,
            )))
            await self.screenshot_error()

    async def privacy_details(self):

        try:
            text = await self._browser.wait_for_list(self._xpath_privacy_details)
            text = text.text
            logger.debug(text)
            return text
        except Exception as error:
            message, session, stacktrace = self.error_parsing(error)
            logger.error(str(dict(
                url=self.url,
                message=message,
                session=session,
                stacktrace=stacktrace,
            )))
            await self.screenshot_error()

    async def title(self) -> str:

        try:
            text = await self._browser.wait_for_list(self._xpath_title)
            text = text.text
            logger.debug(text)
            return text
        except Exception as error:
            message, session, stacktrace = self.error_parsing(error)
            logger.error(str(dict(
                url=self.url,
                message=message,
                session=session,
                stacktrace=stacktrace,
            )))
            await self.screenshot_error()

    @property
    def url(self) -> str:
        return self.url_cleaner(self._url)

    @staticmethod
    def url_cleaner(url: str):
        """simple url cleaner"""
        if not url:
            return
        if url[-1] == '/':
            url = url[:-1]
        return url

    async def visible(self) -> str:

        try:
            text = await self._browser.wait_for_list(self._xpath_visible)
            text = text.text
            logger.debug(text)
            return text
        except Exception as error:
            message, session, stacktrace = self.error_parsing(error)
            logger.error(str(dict(
                url=self.url,
                message=message,
                session=session,
                stacktrace=stacktrace,
            )))
            await self.screenshot_error()

    @staticmethod
    def error_parsing(error, enable_stacktrace: bool = False) -> tuple:
        """parses selenium exeption error"""
        error_parsed = f'{error}'.splitlines()
        error_parsed = [f'{x}'.strip() for x in error_parsed]
        message = error_parsed[0]
        session = None
        stacktrace = None
        if len(error_parsed) > 1:
            session = error_parsed[1]
            stacktrace = error_parsed[2:]
            stacktrace = ' '.join(stacktrace)

        if enable_stacktrace:
            return message, session, stacktrace

        return message, session, 'disabled'

    async def get(self, url: str) -> bool:
        """get url"""

        start = datetime.datetime.now().timestamp()

        result = await self._browser.get(url=url)
        logger.info(str(dict(
            url=url,
            result=result,
        )))
        await self.screenshot()

        end = datetime.datetime.now().timestamp()
        seconds_elapsed = int(end - start)

        logger.info(str(dict(
            seconds_elapsed=seconds_elapsed,
            result=result,
        )))
        self._rate_counter.append(seconds_elapsed)

        return result

    async def get_about(self, rate_limiting: bool = True):
        """get about page"""
        url = f'{self.url}/about'

        if rate_limiting:
            result = await self.get_with_rate_limiter(url=url)
        else:
            result = await self.get(url=url)

        logger.info(str(dict(
            url=url,
            result=result,
        )))
        await self.screenshot()
        return result

    async def get_with_rate_limiter(
            self,
            url: str,
            retry: int = 0,
            retries: int = 5,
            wait_between_retries: int = None,
            rate_per_minute: int = None,
    ) -> bool:
        """get with rate dynamic limit"""
        if wait_between_retries:
            self._wait_between_retries = wait_between_retries

        result = None
        while retry < retries:

            if await self.rate_limited():
                await self.rate_limit_increase()

                self._rate_counter.append(self._wait_between_retries)
                Sleeper.seconds(seconds=self._wait_between_retries)
                logger.error(str(dict(
                    url=url,
                    retry=retry,
                    retries=retries,
                )))
                continue
            else:
                await self.rate_limit_decrease()

            result = await self.get(url=url)
            await self.screenshot()
            logger.info(f'{result}')
            return result

            retry = retry + 1

        logger.error(f'{url}')
        await self.screenshot_error()
        return result

    async def rate_limit_decrease(self, multiplier: int = 0.75):
        before = self._wait_between_retries
        self._wait_between_retries = abs(int(self._wait_between_retries * multiplier))

        if self._wait_between_retries == 0:
            self._wait_between_retries = 1

        logger.info(str(dict(
            before=before,
            after=self._wait_between_retries,
            multiplier=multiplier,
        )))
        return self._wait_between_retries

    async def rate_limit_increase(self, multiplier: int = 2):
        before = self._wait_between_retries
        self._wait_between_retries = abs(int(self._wait_between_retries * multiplier))

        logger.info(str(dict(
            before=before,
            after=self._wait_between_retries,
            multiplier=multiplier,
        )))
        return self._wait_between_retries

    async def rate_limited(self):
        """rate limit checker"""
        if await self.current_rate_too_fast():
            logger.info(True)
            await self.screenshot()
            return True

        if await self.temporarily_blocked() or await self.must_login():
            logger.info(True)
            await self.screenshot()
            return True

        logger.error(False)
        await self.screenshot_error()
        return False

    async def run(self):
        """run selenium browser"""
        if self._browser:
            logger.info(f'{self._browser}')
            return self._browser.run()

    async def reset_rate_counter(self):
        self._rate_counter = []
        logger.info(self._rate_counter)
        return self._rate_counter

    async def restart(self):
        """quit and start new instance of selenium"""
        if self._browser:
            await self.quit()
        logger.info(f'{self._browser}')
        return self.start()

    async def screenshot(self, filename: str = 'screenshot.png'):
        try:
            screenshot = await self._browser.save_screenshot(filename=filename, folder='.')
            logger.debug(f'{screenshot}')
            return screenshot
        except Exception as error:
            raise Exception(error)

    async def screenshot_error(self):
        """get error screenshot"""
        screenshot = await self.screenshot(filename='screenshot-error.png')
        logger.debug(f'{screenshot}')
        return screenshot

    async def screenshot_success(self):
        """get success screenshot"""
        screenshot = await self.screenshot(filename='screenshot-success.png')
        logger.debug(f'{screenshot}')
        return screenshot

    async def set_url(self, url: str) -> str:
        """set new url"""
        self._url = url
        return self.url

    async def start(self, headless: bool = True, random_user_agent: bool = False, set_user_agent: str = None):
        """start new instance of selenium"""
        self._browser.config.webdriver_wrapper = ChromeWrapper()

        if headless:
            await self._browser.config.webdriver_wrapper.enable_headless()
            await self._browser.config.webdriver_wrapper.set_locale_experimental()
        else:
            await self._browser.config.webdriver_wrapper.set_locale_experimental()

        if random_user_agent:
            await self._browser.config.webdriver_wrapper.set_user_agent(
                await self._browser.get_random_user_agent()
            )
        elif set_user_agent:
            await self._browser.config.webdriver_wrapper.set_user_agent(
                set_user_agent
            )

        logger.info(str(dict(
            browser=self._browser
        )))
        browser = await self._browser.run()
        await self._browser.config.webdriver_wrapper.set_window_size(width=1920 * 0.6, height=1080)
        return browser

    async def stop(self):
        """alias to quit"""
        return await self.quit()

    async def to_dict(self):
        return dict(
            content_unavailable=await self.content_unavailable(),
            creation_date=await self.creation_date(),
            creation_date_timestamp=await self.creation_date_timestamp(),
            history=await self.history(),
            members=await self.members(),
            members_count=await self.members_count(),
            posts_monthly=await self.posts_monthly(),
            posts_monthly_count=await self.posts_monthly_count(),
            posts_today=await self.posts_today(),
            posts_today_count=await self.posts_today_count(),
            privacy=await self.privacy(),
            privacy_details=await self.privacy_details(),
            title=await self.title(),
            url=self.url,
            visible=await self.visible(),
        )

    async def quit(self):
        """quit selenium"""
        if self._browser:
            logger.info(f'{self._browser}')
            return await self._browser.quit()



import os
import json
import time
import base64
import datetime
import tempfile
import selenium
import selenium.webdriver

from selenium.webdriver.common.by import By
from selenium.webdriver.common.keys import Keys
from urllib.parse import urlparse
from bs4 import BeautifulSoup

from automon import log
from automon.helpers.dates import Dates
from automon.helpers.sleeper import Sleeper
from automon.helpers.sanitation import Sanitation

from .config import SeleniumConfig
from .user_agents import SeleniumUserAgentBuilder
from .exceptions import *

logger = log.logging.getLogger(__name__)
logger.setLevel(log.DEBUG)


class SeleniumBrowser(object):
    config: SeleniumConfig
    webdriver: selenium.webdriver
    status: int

    def __init__(self, config: SeleniumConfig = None):
        """A selenium wrapper"""

        self._config = config or SeleniumConfig()

    def __repr__(self):
        try:
            return str(dict(
                webdriver=self.webdriver.name or None,
                window_size=self.window_size,
            ))
        except Exception as error:
            pass

        return f'{__class__}'

    @property
    def by(self) -> By:
        """Set of supported locator strategies"""
        return selenium.webdriver.common.by.By()

    @property
    def config(self):
        return self._config

    async def cookie_file_to_dict(self, file: str = 'cookies.txt'):
        logger.debug(f'{file}')
        with open(file, 'r') as file:
            return json.loads(file.read())

    @property
    def _current_url(self):
        try:
            self.webdriver.current_url
            return self.webdriver.current_url
        except Exception as error:
            return

    @property
    def webdriver(self):
        return self.config.webdriver

    @property
    def get_log(self) -> list:
        """Gets the log for a given log type"""
        logs = []
        for log_type in self.webdriver.log_types:
            logs.append(
                {
                    log_type: self.webdriver.get_log(log_type)
                }
            )

        return logs

    @property
    def keys(self):
        """Set of special keys codes"""
        return selenium.webdriver.common.keys.Keys

    async def refresh(self):
        self.webdriver.refresh()
        logger.info(f'{True}')

    @property
    def url(self):
        return self.current_url

    @property
    def user_agent(self):
        try:
            return self.webdriver.execute_script("return navigator.userAgent")
        except:
            return None

    @property
    def current_url(self):
        if self.webdriver:
            logger.debug(self._current_url)
            if self._current_url == 'data:,':
                return ''
            return self._current_url

        logger.info(None)
        return ''

    @property
    def window_size(self):
        return self.config.webdriver_wrapper.window_size

    def _screenshot_name(self, prefix=None):
        """Generate a unique filename"""

        title = self.webdriver.title
        url = self._current_url
        hostname = urlparse(url).hostname

        hostname_ = Sanitation.ascii_numeric_only(hostname)
        title_ = Sanitation.ascii_numeric_only(title)
        timestamp = Dates.filename_timestamp()

        if prefix:
            prefix = Sanitation.safe_string(prefix)
            return f'{prefix}_{hostname_}_{title_}_{timestamp}.png'

        return f'{hostname_}_{title_}_{timestamp}.png'

    async def action_click(
            self,
            xpath: str, **kwargs
    ) -> selenium.webdriver.Chrome.find_element:
        """perform mouse command"""
        try:
            logger.debug(str(dict(
                xpath=xpath,
            )))
            return await self.find_element(value=xpath, by=self.by.XPATH, **kwargs).click()

        except Exception as error:
            raise Exception(error)

    async def action_type(
            self,
            key: str or Keys,
            secret: bool = True,
    ) -> selenium.webdriver.common.action_chains.ActionChains:
        """perform keyboard command"""

        if secret:
            key = f'*' * len(key)

        logger.debug(str(dict(
            send_keys=key,
        )))

        try:
            return selenium.webdriver.common.action_chains.ActionChains(
                self.webdriver).send_keys(key).perform()

        except Exception as error:
            raise Exception(error)

    async def add_cookie(self, cookie_dict: dict) -> bool:
        result = self.webdriver.add_cookie(cookie_dict=cookie_dict)

        if result is None:
            logger.debug(f'{cookie_dict}')
            return True

        logger.error(f'{cookie_dict}')
        return False

    async def add_cookie_from_file(self, file: str) -> bool:
        """add cookies from file"""
        if os.path.exists(file):
            await self.add_cookies_from_list(
                await self.cookie_file_to_dict(file=file)
            )
            return True

        logger.error(f'{file}')
        return False

    async def add_cookies_from_list(self, cookies_list: list) -> bool:
        for cookie in cookies_list:
            await self.add_cookie(cookie_dict=cookie)

        logger.debug(f'{True}')
        return True

    async def add_cookie_from_current_url(self):
        logger.info(f'{self.url}')
        return self.add_cookie_from_url(self.url)

    async def add_cookie_from_url(self, url: str) -> bool:
        """add cookies from matching hostname"""
        cookie_file = await self._url_filename(url=url)

        if os.path.exists(cookie_file):
            logger.info(f'{cookie_file}')
            return self.add_cookie_from_file(file=cookie_file)

        logger.error(f'{cookie_file}')

    async def add_cookie_from_base64(self, base64_str: str) -> bool:
        if base64_str:
            await self.add_cookies_from_list(
                json.loads(base64.b64decode(base64_str))
            )
            logger.debug(f'{True}')
            return True

        logger.error(f'{base64_str}')
        return False

    async def delete_all_cookies(self) -> None:
        result = self.webdriver.delete_all_cookies()
        logger.info(f'{True}')
        return result

    async def _url_filename(self, url: str):
        parsed = await self.urlparse(url)
        hostname = parsed.hostname
        cookie_file = f'cookies-{hostname}.txt'
        logger.info(f'{cookie_file}')
        return cookie_file

    async def get_cookie(self, name: str) -> dict:
        result = self.webdriver.get_cookie(name=name)
        logger.info(f'{result}')
        return result

    async def get_cookies(self) -> [dict]:
        result = self.webdriver.get_cookies()
        logger.debug(f'{True}')
        return result

    async def get_cookies_base64(self) -> base64:
        result = self.get_cookies()
        logger.debug(f'{True}')
        return base64.b64encode(
            json.dumps(result).encode()
        ).decode()

    async def get_cookies_json(self) -> json.dumps:
        cookies = self.get_cookies()
        logger.debug(f'{True}')
        return json.dumps(cookies)

    async def get_cookies_summary(self):
        result = self.get_cookies()
        summary = {}
        if result:
            for cookie in result:
                cookie = dict(cookie)
                domain = cookie.get('domain')
                name = cookie.get('name')
                expiry = cookie.get('expiry')

                if domain in summary.keys():
                    summary[domain].append(cookie)
                else:
                    summary[domain] = [cookie]

        logger.debug(f'{summary}')
        return summary

    async def close(self):
        """close browser"""
        logger.info(f'closed')
        self.webdriver.close()

    @staticmethod
    async def error_parsing(error) -> tuple:
        try:
            error_parsed = f'{error}'.splitlines()
            error_parsed = [f'{x}'.strip() for x in error_parsed]
            message = error_parsed[0]
            session = error_parsed[1]
            stacktrace = error_parsed[2:]
            stacktrace = ' '.join(stacktrace)

            return message, session, stacktrace

        except Exception as error:
            logger.error(error)

        return error, None, None

    async def find_element(
            self,
            value: str,
            by: By.ID = By.ID,
            **kwargs
    ) -> selenium.webdriver.Chrome.find_element:
        """find element"""
        logger.info(str(dict(
            current_url=self.current_url,
            value=value,
        )))
        return self.webdriver.find_element(value=value, by=by, **kwargs)

    async def find_xpath(
            self,
            value: str,
            by: By = By.XPATH,
            **kwargs
    ) -> selenium.webdriver.Chrome.find_element:
        """find xpath"""
        logger.info(str(dict(
            current_url=self.current_url,
            value=value,
        )))
        return self.find_element(value=value, by=by, **kwargs)

    async def get(self, url: str, **kwargs) -> bool:
        """get url"""
        if not self.webdriver:
            logger.error(f'missing webdriver')
            raise Exception(f'missing webdriver')

        try:
            if self.webdriver.get(url, **kwargs) is None:
                logger.info(str(dict(
                    url=url,
                    current_url=self.current_url,
                    kwargs=kwargs
                )))
            return True
        except Exception as error:
            logger.error(str(dict(
                error=error,
            )))

        return False

    async def get_page(self, *args, **kwargs):
        """alias to get"""
        return self.get(*args, **kwargs)

    async def get_page_source(self) -> str:
        """get page source"""
        return self.webdriver.page_source

    async def get_page_source_beautifulsoup(
            self,
            markdup: str = None,
            features: str = 'lxml') -> BeautifulSoup:
        """read page source with beautifulsoup"""
        if not markdup:
            markdup = self.get_page_source()
        return BeautifulSoup(
            markup=markdup,
            features=features)

    async def get_random_user_agent(
            self,
            filter: list or str = None,
            case_sensitive: bool = False) -> str:
        return SeleniumUserAgentBuilder().get_random_agent(
            filter=filter,
            case_sensitive=case_sensitive)

    async def get_screenshot_as_base64(self, **kwargs):
        """screenshot as base64"""
        screenshot = self.webdriver.get_screenshot_as_base64(**kwargs)
        logger.debug(f'{round(len(screenshot) / 1024)} KB')

        return screenshot

    async def get_screenshot_as_file(
            self,
            filename: str = None,
            prefix: str = None,
            folder: str = None,
            **kwargs) -> bool:
        return await self.save_screenshot(
            self,
            filename=filename,
            prefix=prefix,
            folder=folder,
            **kwargs)

    async def get_screenshot_as_png(self, **kwargs):
        """screenshot as png"""
        screenshot = self.webdriver.get_screenshot_as_png(**kwargs)
        logger.debug(f'{round(len(screenshot) / 1024)} KB')

        return screenshot

    async def is_running(self) -> bool:
        """browser is running"""
        if self.webdriver:
            logger.info(f'{True}')
            return True
        logger.error(f'{False}')
        return False

    async def urlparse(self, url: str):
        parsed = urlparse(url=url)
        logger.debug(f'{parsed}')
        return parsed

    async def quit(self) -> bool:
        """gracefully quit browser"""
        try:
            self.webdriver.close()
            self.webdriver.quit()
            self.webdriver.stop_client()
        except Exception as error:
            message, session, stacktrace = await self.error_parsing(error)
            logger.error(str(dict(
                message=message,
                session=session,
                stacktrace=stacktrace,
            )))
            return False
        return True

    async def run(self):
        """run browser"""
        try:
            await self.config.run()
        except:
            return False

    async def save_cookies_for_current_url(self):
        filename = self._url_filename(url=self.url)
        logger.info(f'{filename}')
        return await self.save_cookies_to_file(file=filename)

    async def save_cookies_to_file(self, file: str):
        with open(file, 'w') as cookies:
            cookies.write(
                await self.get_cookies_json()
            )

        if os.path.exists(file):
            logger.info(f'{os.path.abspath(file)} ({os.stat(file).st_size} B)')
            return True

        logger.error(f'{file}')
        return False

    async def save_screenshot(
            self,
            filename: str = None,
            prefix: str = None,
            folder: str = None,
            **kwargs) -> bool:
        """save screenshot to file"""

        if not filename:
            filename = self._screenshot_name(prefix)

        if not folder:
            path = os.path.abspath(tempfile.gettempdir())
        else:
            path = os.path.abspath(folder)

        if not os.path.exists(path):
            os.makedirs(path)

        save = os.path.join(path, filename)

        if self.webdriver.save_screenshot(save, **kwargs):
            logger.info(f'Saving screenshot to: {save} ({round(os.stat(save).st_size / 1024)} KB)')
            return True

        return False

    async def set_window_size(self, width=1920, height=1080, device_type=None) -> bool:
        """set browser resolution"""

        try:
            self.config.webdriver_wrapper.set_window_size(
                width=width,
                height=height,
                device_type=device_type)
        except Exception as error:
            message, session, stacktrace = await self.error_parsing(error)
            logger.error(str(dict(
                message=message,
                session=session,
                stacktrace=stacktrace,
            )))
            return False
        return True

    async def set_window_position(self, x: int = 0, y: int = 0):
        """set browser position"""
        result = self.webdriver.set_window_position(x, y)
        logger.info(f'{result}')
        return result

    async def start(self):
        """alias to run"""
        return self.run()

    async def wait_for(
            self,
            value: str,
            by: By = By.XPATH,
            timeout: int = 1,
            **kwargs
    ) -> selenium.webdriver.Chrome.find_element:
        """wait for an element"""
        timeout_start = time.time()
        timeout_elapsed = round(abs(timeout_start - time.time()), 1)

        while timeout_elapsed < timeout:

            logger.debug(str(dict(
                timeout=f'{timeout_elapsed}/{timeout}',
                by=by,
                current_url=self.current_url,
                value=value,
            )))

            try:
                return await self.find_element(
                    by=by,
                    value=value,
                    **kwargs)
            except Exception as error:
                logger.error(error)

            timeout_elapsed = round(abs(timeout_start - time.time()), 1)

        raise NoSuchElementException(value)

    async def wait_for_list(
            self,
            values: list,
            by: By = By.XPATH,
            timeout: int = 1,
            **kwargs
    ) -> selenium.webdriver.Chrome.find_element:
        """wait for a list of elements"""
        if isinstance(values, list):
            for value in values:

                logger.debug(str(dict(
                    checking=f'{values.index(value) + 1}/{len(values)}',
                    value=value,
                )))

                try:
                    return await self.wait_for(
                        value=value,
                        by=by,
                        timeout=timeout,
                        **kwargs,
                    )
                except Exception as error:
                    logger.error(error)

        raise NoSuchElementException(values)

    async def wait_for_element(
            self,
            element: str or list,
            timeout: int = 1,
            **kwargs
    ) -> selenium.webdriver.Chrome.find_element:
        """wait for an element"""
        return await self.wait_for(
            value=element,
            by=self.by.ID,
            timeout=timeout,
            **kwargs)

    async def wait_for_xpath(
            self,
            xpath: str or list,
            timeout: int = 1,
            **kwargs) -> str or False:
        """wait for an xpath"""
        return await self.wait_for(
            value=xpath,
            by=self.by.XPATH,
            timeout=timeout,
            **kwargs)
